@@ -23,9 +23,9 @@ else:
     from django.test import TestCase, TransactionTestCase
     from django.test.runner import default_test_processes
     from django.test.selenium import SeleniumTestCaseBase
-    from django.test.utils import get_runner
+    from django.test.utils import NullTimeKeeper, TimeKeeper, get_runner
     from django.utils.deprecation import (
-        RemovedInDjango31Warning, RemovedInDjango40Warning,
+        RemovedInDjango40Warning, RemovedInDjango41Warning,
     )
     from django.utils.log import DEFAULT_LOGGING
     from django.utils.version import PY37
@@ -40,11 +40,19 @@ else:
 
 # Make deprecation warnings errors to ensure no usage of deprecated features.
 warnings.simplefilter("error", RemovedInDjango40Warning)
-warnings.simplefilter('error', RemovedInDjango31Warning)
-# Make runtime warning errors to ensure no usage of error prone patterns.
+warnings.simplefilter('error', RemovedInDjango41Warning)
+# Make resource and runtime warning errors to ensure no usage of error prone
+# patterns.
+warnings.simplefilter("error", ResourceWarning)
 warnings.simplefilter("error", RuntimeWarning)
 # Ignore known warnings in test dependencies.
 warnings.filterwarnings("ignore", "'U' mode is deprecated", DeprecationWarning, module='docutils.io')
+# RemovedInDjango41Warning: Ignore MemcachedCache deprecation warning.
+warnings.filterwarnings(
+    'ignore',
+    'MemcachedCache is deprecated',
+    category=RemovedInDjango41Warning,
+)
 
 RUNTESTS_DIR = os.path.abspath(os.path.dirname(__file__))
 
@@ -88,8 +96,9 @@ ALWAYS_MIDDLEWARE = [
 # avoid "RuntimeError: Model class X doesn't declare an explicit app_label
 # and isn't in an application in INSTALLED_APPS."
 CONTRIB_TESTS_TO_APPS = {
-    'flatpages_tests': 'django.contrib.flatpages',
-    'redirects_tests': 'django.contrib.redirects',
+    'deprecation': ['django.contrib.flatpages', 'django.contrib.redirects'],
+    'flatpages_tests': ['django.contrib.flatpages'],
+    'redirects_tests': ['django.contrib.redirects'],
 }
 
 
@@ -182,6 +191,7 @@ def setup(verbosity, test_labels, parallel, start_at, start_after):
     settings.LOGGING = log_config
     settings.SILENCED_SYSTEM_CHECKS = [
         'fields.W342',  # ForeignKey(unique=True) -> OneToOneField
+        'fields.W903',  # NullBooleanField deprecated.
     ]
 
     # Load all the ALWAYS_INSTALLED_APPS.
@@ -225,7 +235,9 @@ def setup(verbosity, test_labels, parallel, start_at, start_after):
         )
 
         if module_name in CONTRIB_TESTS_TO_APPS and module_found_in_labels:
-            settings.INSTALLED_APPS.append(CONTRIB_TESTS_TO_APPS[module_name])
+            for contrib_app in CONTRIB_TESTS_TO_APPS[module_name]:
+                if contrib_app not in settings.INSTALLED_APPS:
+                    settings.INSTALLED_APPS.append(contrib_app)
 
         if module_found_in_labels and module_label not in installed_app_names:
             if verbosity >= 2:
@@ -242,6 +254,10 @@ def setup(verbosity, test_labels, parallel, start_at, start_after):
 
     apps.set_installed_apps(settings.INSTALLED_APPS)
 
+    # Set an environment variable that other code may consult to see if
+    # Django's own test suite is running.
+    os.environ['RUNNING_DJANGOS_TEST_SUITE'] = 'true'
+
     return state
 
 
@@ -255,6 +271,7 @@ def teardown(state):
     # FileNotFoundError at the end of a test run (#27890).
     from multiprocessing.util import _finalizer_registry
     _finalizer_registry.pop((-100, 0), None)
+    del os.environ['RUNNING_DJANGOS_TEST_SUITE']
 
 
 def actual_test_processes(parallel):
@@ -284,15 +301,13 @@ class ActionSelenium(argparse.Action):
 
 def django_tests(verbosity, interactive, failfast, keepdb, reverse,
                  test_labels, debug_sql, parallel, tags, exclude_tags,
-                 test_name_patterns, start_at, start_after, pdb):
+                 test_name_patterns, start_at, start_after, pdb, buffer,
+                 timing):
     state = setup(verbosity, test_labels, parallel, start_at, start_after)
-    extra_tests = []
-
     # Run the test suite, including the extra validation tests.
     if not hasattr(settings, 'TEST_RUNNER'):
         settings.TEST_RUNNER = 'django.test.runner.DiscoverRunner'
     TestRunner = get_runner(settings)
-
     test_runner = TestRunner(
         verbosity=verbosity,
         interactive=interactive,
@@ -305,11 +320,10 @@ def django_tests(verbosity, interactive, failfast, keepdb, reverse,
         exclude_tags=exclude_tags,
         test_name_patterns=test_name_patterns,
         pdb=pdb,
+        buffer=buffer,
+        timing=timing,
     )
-    failures = test_runner.run_tests(
-        test_labels or get_installed(),
-        extra_tests=extra_tests,
-    )
+    failures = test_runner.run_tests(test_labels or get_installed())
     teardown(state)
     return failures
 
@@ -500,6 +514,14 @@ if __name__ == "__main__":
         '--pdb', action='store_true',
         help='Runs the PDB debugger on error or failure.'
     )
+    parser.add_argument(
+        '-b', '--buffer', action='store_true',
+        help='Discard output of passing tests.',
+    )
+    parser.add_argument(
+        '--timing', action='store_true',
+        help='Output timings, including database set up and total run time.',
+    )
     if PY37:
         parser.add_argument(
             '-k', dest='test_name_patterns', action='append',
@@ -560,13 +582,17 @@ if __name__ == "__main__":
             options.start_at, options.start_after,
         )
     else:
-        failures = django_tests(
-            options.verbosity, options.interactive, options.failfast,
-            options.keepdb, options.reverse, options.modules,
-            options.debug_sql, options.parallel, options.tags,
-            options.exclude_tags,
-            getattr(options, 'test_name_patterns', None),
-            options.start_at, options.start_after, options.pdb,
-        )
+        time_keeper = TimeKeeper() if options.timing else NullTimeKeeper()
+        with time_keeper.timed('Total run'):
+            failures = django_tests(
+                options.verbosity, options.interactive, options.failfast,
+                options.keepdb, options.reverse, options.modules,
+                options.debug_sql, options.parallel, options.tags,
+                options.exclude_tags,
+                getattr(options, 'test_name_patterns', None),
+                options.start_at, options.start_after, options.pdb, options.buffer,
+                options.timing,
+            )
+        time_keeper.print_results()
         if failures:
             sys.exit(1)
